﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using System.IO;
using RecordingLoaders;
using RecordingInterface;

namespace PnrfReader
{
     public partial class Form1 : Form
     {
          public Form1()
          {
               InitializeComponent();
          }

          private void btnFolderSelect_Click(object sender, EventArgs e)
          {
               OpenFileDialog Dialog = new OpenFileDialog();
               Dialog.Title = "Select Recording";
               Dialog.Filter = "Recording Files (pnrf)|*.pnrf";
               if (edtSourceFileName.Text != "")
               {
                    if (File.Exists(edtSourceFileName.Text))
                    {
                         Dialog.FileName = edtSourceFileName.Text;
                    }
               }

               if (Dialog.ShowDialog(this) == DialogResult.OK)
               {
                    String myFileName = Dialog.FileName;
                    if (myFileName.Length != 0)
                    {
                         if (File.Exists(myFileName))
                         {
                              try
                              {
                                   edtSourceFileName.Text = myFileName;
                              }
                              catch
                              {
                                   MessageBox.Show("Error reading File");
                              }
                         }
                    }
               }
          }

          private void btnLoad_Click(object sender, EventArgs e)
          {
               if (!File.Exists(edtSourceFileName.Text))
               {
                    MessageBox.Show("File " + edtSourceFileName.Text + " does not exist!");
                    return;
               }
               ShowInfoData();
               ShowRecordingData();
          }

          private void ShowInfoData()
          {
               listBox2.Items.Clear();
               PNRFLoader FromDisk = new PNRFLoader();

               string strRecordingFileName = edtSourceFileName.Text;
               IRecording myData = FromDisk.LoadRecording(strRecordingFileName);

               if (myData == null) return;
               if (myData.DataValues == null) return;

               listBox2.Items.Add(string.Format("D a t a V a l u e s ({0})", myData.DataValues.Count));
               for (int i = 1; i <= myData.DataValues.Count; i++)
               {
                    IDataValue DataVal = myData.DataValues[i];
                    listBox2.Items.Add(string.Format("Name: {0}, Value: {1} {2}, Type: {3}", DataVal.Name, DataVal.Value, DataVal.Units, DataVal.DataType));
               }

               listBox2.Items.Add(string.Empty);
               listBox2.Items.Add(string.Format("R e c o r d e r s ({0})", myData.Recorders.Count));
               for (int i = 1; i <= myData.Recorders.Count; i++)
               {
                    IDataRecorder myDataRecorder = myData.Recorders[i];
                    string cPhysicalName = myDataRecorder.PhysicalName;
                    string cGroup = myDataRecorder.Group.Name;

                    int nTriggers = 0;
                    if (myDataRecorder.Triggers != null)
                    {
                         nTriggers = myDataRecorder.Triggers.Count;
                    }

                    listBox2.Items.Add(string.Format("Recorder: {0} Group: {1} Triggers: {2}", cPhysicalName, cGroup, nTriggers));
               }
          }

          private void ShowRecordingData()
          {
               PNRFLoader FromDisk = new PNRFLoader();

               string strRecordingFileName = edtSourceFileName.Text;
               IRecording myData = FromDisk.LoadRecording(strRecordingFileName);
               lblOutput.Text = myData.Title;
               listBox1.Items.Clear();

               if (myData.Channels.Count < 1)
               {
                    MessageBox.Show("No Data");
                    return;
               }
               IDataChannel myChannel = myData.Channels[1];
               listBox1.Items.Add(string.Format("Recording: {0}", myChannel.Recording.Title));
               listBox1.Items.Add(string.Format("Recorder: {0} Channel: {1}", myChannel.Recorder.Name, myChannel.Name));
               IDataSrc myDataSrc = myChannel.get_DataSource(DataSourceSelect.DataSourceSelect_Mixed);

               double dStartTime = myDataSrc.Sweeps.StartTime;
               double dEndTime = myDataSrc.Sweeps.EndTime;

               listBox1.Items.Add(string.Format("Recording Start: {0}, End: {1}", dStartTime, dEndTime));

               listBox1.Items.Add("");
               object mySegmentsData = null;
               myDataSrc.Data(dStartTime, dEndTime, out mySegmentsData);

               if (mySegmentsData == null)
               {
                    MessageBox.Show("No Data");
                    return;
               }

               IDataSegments mySegments = mySegmentsData as IDataSegments;
               if (mySegments == null)
               {
                    MessageBox.Show("No Segments");
                    return;
               }
               if (mySegments.Count < 1)
               {
                    MessageBox.Show("No Segments");
                    return;
               }

               IDataSegment mySegment = mySegments[1];

               object oRawData;
               mySegment.Waveform(DataSourceResultType.DataSourceResultType_Double64, 1, mySegment.NumberOfSamples, 1, out oRawData);

               if (oRawData == null)
               {
                    MessageBox.Show("No Raw Data");
                    return;
               }

               double[] aSamples = oRawData as double[];

               double X0 = mySegment.StartTime;
               double DeltaX = mySegment.SampleInterval;
               double X, Y;

               for (int i = 0; i < aSamples.Length; i++)
               {
                    X = X0 + i * DeltaX;
                    Y = aSamples[i];
                    listBox1.Items.Add(string.Format("{0:000}:  X = {1}, Y = {2}", i, X, Y));

                    // Do not show more then 500 samples
                    if (i >= 500)
                         break;
               }
          }
     }
}
